@@ -4,20 +4,12 @@
 #  --> https://CRAN.R-project.org/package=spam              #
 #  --> https://CRAN.R-project.org/package=spam64            #
 #  --> https://git.math.uzh.ch/reinhard.furrer/spam         #
-# by Reinhard Furrer [aut, cre], Florian Gerber [ctb],      #
-#    Daniel Gerber [ctb], Kaspar Moesinger [ctb],           #
-#    Youcef Saad [ctb] (SPARSEKIT),                         #
-#    Esmond G. Ng [ctb] (Fortran Cholesky routines),        #
-#    Barry W. Peyton [ctb] (Fortran Cholesky routines),     #
-#    Joseph W.H. Liu [ctb] (Fortran Cholesky routines),     #
-#    Alan D. George [ctb] (Fortran Cholesky routines),      #
-#    Esmond G. Ng [ctb] (Fortran Cholesky routines),        #
-#    Barry W. Peyton [ctb] (Fortran Cholesky routines),     #
-#    Joseph W.H. Liu [ctb] (Fortran Cholesky routines),     #
-#    Alan D. George [ctb] (Fortran Cholesky routines)       #
+# by Reinhard Furrer [aut, cre], Florian Gerber [aut],      #
+#    Roman Flury [aut], Daniel Gerber [ctb],                #
+#    Kaspar Moesinger [ctb]                                 #
 # HEADER END ################################################
 
-     
+
 
 # `"Ops"":
 #      `"+"", `"-"", `"*"", `"/"", `"^"", `"%%"", `"%/%""
@@ -48,13 +40,13 @@ setMethod("!",signature(x="spam"),    function(x){
         x
     } else {
         inefficiencywarning( gettextf("This %s operation may be inefficient",sQuote(.Generic)), prod(dim(x)))
-        spam(as.double( callGeneric(as.matrix(x))), nrow=nrow(x))
+        spam(as.double( callGeneric(as.matrix(x))), nrow=dim(x)[1])
     }
 })
-          
-setMethod("+",signature(e1="spam",e2="missing"), function(e1) e1 )                
-setMethod("-",signature(e1="spam",e2="missing"), function(e1) { e1@entries <- -e1@entries; e1} )                
-              
+
+setMethod("+",signature(e1="spam",e2="missing"), function(e1) e1 )
+setMethod("-",signature(e1="spam",e2="missing"), function(e1) { e1@entries <- -e1@entries; e1} )
+
 #     `Math2" :
 
 setMethod("Math2",signature(x = "spam", digits = "ANY"),
@@ -68,7 +60,7 @@ setMethod("Math","spam", function(x){
         x
     }else{
         x@entries <- callGeneric(x@entries)
-        as.spam.spam( x)  
+        as.spam.spam( x)
     }
 })
 
@@ -88,7 +80,7 @@ setMethod("exp","spam", spam_Math )
 setMethod("log10","spam", spam_Math )
 setMethod("log2","spam", spam_Math )
 # from ?log: Do not set S4 methods on `logb" itself.
-# special case to set base...          
+# special case to set base...
 setMethod("log","spam", function(x,...) {
     if(getOption("spam.structurebased")) {
         x@entries <- callGeneric(x@entries,...)
@@ -100,7 +92,7 @@ setMethod("log","spam", function(x,...) {
           )
 
 
-          
+
 setMethod("cos","spam", spam_Math )
 #setMethod("cospi","spam", spam_Math )
 setMethod("cosh","spam", spam_Math )
@@ -121,29 +113,29 @@ setMethod("cumsum","spam", spam_Math )
 #     `Summary" :
 setMethod("Summary","spam", function(x,...,na.rm=FALSE){
     if(getOption("spam.structurebased")) {
-        callGeneric(x@entries,...,na.rm=na.rm) 
+        callGeneric(x@entries,...,na.rm=na.rm)
     }else{
         if ( prod( x@dimension) == length( x@entries)) {
-            callGeneric(x@entries,...,na.rm=na.rm) 
+            callGeneric(x@entries,...,na.rm=na.rm)
         } else {
             callGeneric(c(0,x@entries),...,na.rm=na.rm)
         }
     }
 }
-          )          
+          )
 
 logical_Summary <- function( x,...,na.rm=FALSE){
     if(getOption("spam.structurebased")) {
-        callGeneric(as.logical(x@entries),...,na.rm=na.rm) 
+        callGeneric(as.logical(x@entries),...,na.rm=na.rm)
     }else{
         if ( prod( x@dimension) == length( x@entries)) {
-            callGeneric(as.logical(x@entries),...,na.rm=na.rm) 
+            callGeneric(as.logical(x@entries),...,na.rm=na.rm)
         } else {
             callGeneric(as.logical(c(0,x@entries)),...,na.rm=na.rm)
         }
     }
 }
-      
+
 
 setMethod("any","spam", logical_Summary)
 setMethod("all","spam", logical_Summary)
@@ -155,16 +147,16 @@ setMethod("all","spam", logical_Summary)
 
 
 #     `Logic" `"&"", `"|"".
-        
-                                       
+
+
 "spam_Logic_vectorspam" <- function(e1, e2) {
     if(getOption("spam.structurebased")) {
+        if( isTRUE(all.equal(length(e2@entries), prod(e2@dimension))))  # spam is full
+            return( as.spam( callGeneric(e1, as.matrix(e2))) )          # 3.0: no as.spam -> matrix
         if(identical(length(e1),1L) | identical(length(e1), length(e2@entries))) {
             e2@entries <- as.double( callGeneric(e1, e2@entries))
             return(e2)
         }
-        if( length(e1) == prod(e2@dimension))
-            return( as.spam( callGeneric(e1, as.matrix(e2))) )
         stop(gettextf("incompatible lengths for %s operation.", sQuote(.Generic)))
     }  else {
         inefficiencywarning( gettextf("This %s operation may be inefficient",sQuote(.Generic)), prod(dim(e2)))
@@ -174,43 +166,43 @@ setMethod("all","spam", logical_Summary)
 
 "spam_Logic_spamvector" <- function(e1, e2)  {
     if(getOption("spam.structurebased")) {
+        if( isTRUE(all.equal(length(e1@entries), prod(e1@dimension))))  # spam is full
+            return( as.spam( callGeneric(as.matrix(e1), e2)) )   # 3.0: no as.spam -> matrix
         if(identical(length(e2),1L) | identical(length(e2), length(e1@entries))) {
             e1@entries <- as.double( callGeneric(e1@entries, e2))
             return(e1)
         }
-        if( length(e2)== prod(e1@dimension))
-            return( as.spam( callGeneric(as.matrix(e1), e2)) )
         stop(gettextf("incompatible lengths for %s operation.", sQuote(.Generic)))
     }  else {
         inefficiencywarning( gettextf("This %s operation may be inefficient",sQuote(.Generic)), prod(dim(e1)))
         return( as.spam( callGeneric(as.matrix(e1), e2)) )
     }
 }
-        
 
-setMethod("|",signature(e1="spam",e2="spam"), 
-          function(e1,e2){ z <- spam_add(e1,e2);z@entries <- rep(1,length(z@colindices));z})
 
-setMethod("&",signature(e1="spam",e2="spam"), 
+setMethod("|",signature(e1="spam",e2="spam"),
+          function(e1,e2){ z <- spam_add(e1,e2);  z@entries <- rep(1,length(z@colindices));z})
+
+setMethod("&",signature(e1="spam",e2="spam"),
           function(e1,e2){ z <- spam_mult(e1,e2); z@entries <- rep(1,length(z@colindices));z})
 setMethod("Logic",signature(e1="spam",e2="vector"), spam_Logic_spamvector)
 setMethod("Logic",signature(e1="vector",e2="spam"), spam_Logic_vectorspam)
 
 ##################################################################################################
-#     `Compare" `"=="", `">"", `"<"", `"!="", `"<="", `">=""                                     
+#     `Compare" `"=="", `">"", `"<"", `"!="", `"<="", `">=""
 "spam_Compare" <- function(e1,e2) {
     inefficiencywarning( gettextf("This %s operation may be inefficient",sQuote(.Generic)),  max(prod(dim(e1)), prod(dim(e2))))
-    as.spam( callGeneric( as.matrix(e1), as.matrix(e2))  )            
+    as.spam( callGeneric( as.matrix(e1), as.matrix(e2))  )
 }
 
 "spam_Compare_spamvector" <- function(e1, e2){
     if(getOption("spam.structurebased")) {
+        if( isTRUE(all.equal(length(e1@entries), prod(e1@dimension))))  # spam is full
+            return( as.spam( callGeneric(as.matrix(e1), e2)) )   # 3.0: no as.spam -> matrix
         if(identical(length(e2),1L) | identical(length(e2), length(e1@entries))) {
             e1@entries <- as.double(callGeneric(e1@entries, e2))
             return(e1)
         }
-        if( length(e2)== prod(e1@dimension))
-            return( as.spam( callGeneric(as.matrix(e1), e2)) )
         stop(gettextf("incompatible lengths for %s operation.", sQuote(.Generic)))
     }  else {
         inefficiencywarning( gettextf("This %s operation may be inefficient",sQuote(.Generic)),   prod(dim(e1)))
@@ -219,12 +211,12 @@ setMethod("Logic",signature(e1="vector",e2="spam"), spam_Logic_vectorspam)
 }
 "spam_Compare_vectorspam" <- function(e1, e2) {
     if(getOption("spam.structurebased")) {
+        if( isTRUE(all.equal(length(e2@entries), prod(e2@dimension))))  # spam is full
+            return( as.spam( callGeneric(e1, as.matrix(e2))) )          # 3.0: no as.spam -> matrix
         if(identical(length(e1),1L) | identical(length(e1), length(e2@entries))) {
             e2@entries <- as.double( callGeneric(e1, e2@entries))
             return(e2)
         }
-        if( length(e1) == prod(e2@dimension))
-            return( as.spam( callGeneric(e1, as.matrix(e2))) )
         stop(gettextf("incompatible lengths for %s operation.", sQuote(.Generic)))
      }  else {
         inefficiencywarning( gettextf("This %s operation may be inefficient",sQuote(.Generic)),   prod(dim(e2)))
@@ -239,9 +231,11 @@ setMethod("Compare",signature(e1="vector",e2="spam"), spam_Compare_vectorspam )
 ##################################################################################################
 #     `Arith": `"+"", `"-"", `"*"", `"^"", `"%%"", `"%/%"", `"/""
 
-"spam_Arith_vectorspam" <- function(e1, e2){    
+"spam_Arith_vectorspam" <- function(e1, e2){
 #    cat("spam_Arith_vectorspam")
     if(getOption("spam.structurebased")) {
+        if( isTRUE(all.equal(length(e2@entries), prod(e2@dimension))))  # spam is full
+            return( as.spam( callGeneric(e1, as.matrix(e2))) )          # 3.0: no as.spam -> matrix
         if(identical(length(e1),1L) | identical(length(e1), length(e2@entries))) {
             e2@entries <- callGeneric(e1, e2@entries)
             return(e2)
@@ -257,12 +251,12 @@ setMethod("Compare",signature(e1="vector",e2="spam"), spam_Compare_vectorspam )
 "spam_Arith_spamvector" <- function(e1, e2){
 #    cat("spam_Arith_spamvector")
     if(getOption("spam.structurebased")) {
+        if( isTRUE(all.equal(length(e1@entries), prod(e1@dimension))))  # spam is full
+            return( as.spam( callGeneric(as.matrix(e1), e2)) )          # 3.0: no as.spam -> matrix
         if(identical(length(e2),1L) | identical(length(e2), length(e1@entries))) {
             e1@entries <- callGeneric(e1@entries, e2)
             return(e1)
         }
-        if( length(e2)== prod(e1@dimension))
-            return( as.spam( callGeneric(as.matrix(e1), e2)) )
         stop(gettextf("incompatible lengths for %s operation.", sQuote(.Generic)))
      }  else {
         inefficiencywarning( gettextf("This %s operation may be inefficient",sQuote(.Generic)), prod(dim(e1)))
@@ -273,8 +267,8 @@ spam_Arith <- function(e1,e2) {
     inefficiencywarning( gettextf("This %s operation may be inefficient",sQuote(.Generic)),  max(prod(dim(e1)), prod(dim(e2))))
     as.spam( callGeneric( as.matrix(e1), as.matrix(e2)))
 }
-        
- 
+
+
 
 setMethod("Arith",signature(e1="spam",e2="spam"),   spam_Arith )
 setMethod("Arith",signature(e1="spam",e2="vector"), spam_Arith_spamvector)
@@ -293,7 +287,7 @@ setMethod("^",signature(e1="spam",e2="spam"), function(e1,e2){ "^"(e1,as.matrix(
 #######################################################################
 "spam_add" <- function(A, B, s=1)
 {
-    
+
                                         # cat("spam_add")
     nrow <- A@dimension[1]
     ncol <- A@dimension[2]
@@ -306,19 +300,19 @@ setMethod("^",signature(e1="spam",e2="spam"), function(e1,e2){ "^"(e1,as.matrix(
         SS <- .format32
 
     nzmax <- .C64("aplbdg",
-                  ## subroutine aplbdg (nrow,ncol,ja,ia,jb,ib,ndegr,nnz,iw) 
+                  ## subroutine aplbdg (nrow,ncol,ja,ia,jb,ib,ndegr,nnz,iw)
                   SIGNATURE = c(SS$signature, SS$signature,
                       SS$signature, SS$signature, SS$signature, SS$signature,
                       SS$signature, SS$signature, SS$signature),
-                  
+
                   nrow,
                   ncol,
-                  
+
                   A@colindices,
                   A@rowpointers,
                   B@colindices,
                   B@rowpointers,
-                  
+
                   vector_dc(SS$type, nrow),
                   nnz = vector_dc(SS$type, 1),
                   vector_dc(SS$type, ncol),
@@ -337,24 +331,24 @@ setMethod("^",signature(e1="spam",e2="spam"), function(e1,e2){ "^"(e1,as.matrix(
                   "double", SS$signature, SS$signature,
                   "double", SS$signature, SS$signature,
                   SS$signature, SS$signature),
-              
+
               nrow,
               ncol,
-              
+
               A@entries,
               A@colindices,
               A@rowpointers,
-              
+
               s,
-              
+
               B@entries,
               B@colindices,
               B@rowpointers,
-              
+
               entries     = vector_dc("double", nzmax),
               colindices  = vector_dc(SS$type, nzmax),
               rowpointers = vector_dc(SS$type, nrow+1),
-              
+
               nzmax+1,
               ierr = vector_dc(SS$type, 1),
 
@@ -417,7 +411,7 @@ setMethod("-",signature(e1="spam",e2="spam"),  function(e1,e2){ spam_add(e1, e2,
 #    e2 <- as.spam(e2)
 #  if(!(is.spam(e1) && is.spam(e2)))
 #    stop("Arguments must be of class:  vector, matrix or spam")
-    
+
   e1row <- e1@dimension[1]
   e1col <- e1@dimension[2]
   if(e1col != e2@dimension[2] | e1row != e2@dimension[1])
@@ -438,18 +432,18 @@ setMethod("-",signature(e1="spam",e2="spam"),  function(e1,e2){ spam_add(e1, e2,
                 "double", SS$signature, SS$signature,
                 "double", SS$signature, SS$signature,
                 SS$signature, SS$signature),
-            
+
             e1row,
             e1col,
-            
+
             e1@entries, e1@colindices, e1@rowpointers,
-            
+
             e2@entries, e2@colindices, e2@rowpointers,
-            
+
             entries     = vector_dc("double", nnzmax),
             colindices  = vector_dc(SS$type, nnzmax),
             rowpointers = vector_dc(SS$type, e1row+1),
-            
+
             nnzmax,
             ierr = vector_dc(SS$type,1),
 
@@ -473,11 +467,11 @@ setMethod("-",signature(e1="spam",e2="spam"),  function(e1,e2){ spam_add(e1, e2,
   ##               as.integer(nnzmax),
   ##               ierr = vector("integer",1),
   ##               NAOK=getOption("spam.NAOK"),  PACKAGE = "spam")
-    
+
     if(z$ierr != 0)      stop("insufficient space for element-wise sparse matrix multiplication")
 
     nnz <- z$rowpointers[e1row+1]-1
-    
+
     if(identical(z$entries,0L)){#trap zero matrix
         z$colindices <- 1L
         z$rowpointers <- c(1L,rep(2L,e1row))
@@ -517,12 +511,12 @@ setMethod("*",signature(e1="spam",e2="spam"), spam_mult)
         SS <- .format64()
     else
         SS <- .format.spam(A)
-    
+
     return(array(
         .C64("addsparsefull",
                  SIGNATURE = c(SS$signature, "double", SS$signature, SS$signature,
                                "double"),
-                 
+
                  nrow,
                  A@entries,
                  A@colindices,
@@ -562,20 +556,20 @@ setMethod("*",signature(e1="spam",e2="spam"), spam_mult)
         SS <- .format64()
     else
         SS <- .format.spam(A)
-    
+
     return(array(
         .C64("subfullsparse",
              SIGNATURE = c( SS$signature, SS$signature,
                            "double", SS$signature, SS$signature,
                            "double"),
-             
+
              nrow,
              ncol,
-             
+
              A@entries,
              A@colindices,
              A@rowpointers,
-             
+
              b = c(B),
 
              INTENT = c("r", "r",
@@ -603,7 +597,7 @@ setMethod("*",signature(e1="spam",e2="spam"), spam_mult)
 #    } else  B <- rep(B,pdim %/% length(B))
 #  }
 #  if (!is.double(B[1]))  B <- as.double(B)
-  
+
     if(  getOption("spam.force64") )
         SS <- .format64()
     else
@@ -613,12 +607,12 @@ setMethod("*",signature(e1="spam",e2="spam"), spam_mult)
         .C64("subsparsefull",
              SIGNATURE = c(SS$signature, "double", SS$signature, SS$signature,
                            "double"),
-             
+
              nrow,
              A@entries,
              A@colindices,
              A@rowpointers,
-             
+
              b = c(B),
 
              INTENT = c("r", "r", "r", "r",
@@ -633,7 +627,7 @@ setMethod("+",signature(e1="matrix",   e2="spam"), function(e1,e2){ matrix_add_s
 setMethod("-",signature(e1="matrix",   e2="spam"), function(e1,e2){ matrix_sub_matrixspam(e1,e2)})
 setMethod("-",signature(e1="spam",   e2="matrix"), function(e1,e2){ matrix_sub_spammatrix(e1,e2)})
 
-                                      
+
 #"spam_division" <- function(e1,e2) { # Element-wise matrix division of two spams
 #  if(is.numeric(e1) && length(e1) == 1)
 #  { e2@entries <- e1/e2@entries
@@ -671,9 +665,9 @@ setMethod("-",signature(e1="spam",   e2="matrix"), function(e1,e2){ matrix_sub_s
 #        return(new("spam",entries=z$entries[1:nz],colindices=z$colindices[1:nz],rowpointers=z$rowpointers,
 #                   dimension=c(nrow,ncol)))
 #    }
- 
 
-   
+
+
 ##"spam_exponent" <- function(e1, e2)
 #{
 #    nrow <- e1@dimension[1]
